from functools import partial
from unittest import TextTestResult, TextTestRunner

import httpretty
from django.test.runner import DiscoverRunner, RemoteTestResult

from .service import BlipService

blip_obj = BlipService()


class BlipTestRunnerRemoteTestResult(RemoteTestResult):
    def startTest(self, test):
        super().startTest(test)
        httpretty.reset()
        if blip_obj.blip_silently_bypass:
            blip_obj.register_httpretty_uri()
        httpretty.enable(allow_net_connect=False, verbose=blip_obj.blip_verbose)

    def stopTest(self, test):
        super().stopTest(test)
        httpretty.disable()
        httpretty.reset()


class BlipTextTestResult(TextTestResult):
    def __init__(self, stream, descriptions, verbosity):
        super(BlipTextTestResult, self).__init__(stream, descriptions, verbosity)

    def startTest(self, test):
        super(BlipTextTestResult, self).startTest(test)
        httpretty.reset()
        if blip_obj.blip_silently_bypass:
            blip_obj.register_httpretty_uri()
        httpretty.enable(allow_net_connect=False, verbose=blip_obj.blip_verbose)

    def stopTest(self, test):
        super(BlipTextTestResult, self).stopTest(test)
        httpretty.disable()
        httpretty.reset()


class BlipTextTestRunner(TextTestRunner):
    resultclass = BlipTextTestResult


class BlipTestRunner(DiscoverRunner):
    def __init__(self, **kwargs):
        super(BlipTestRunner, self).__init__(**kwargs)
        self.parallel_test_suite.runner_class = partial(  # Add this
            self.parallel_test_suite.runner_class,
            resultclass=BlipTestRunnerRemoteTestResult,
        )

    test_runner = BlipTextTestRunner



import setuptools

setuptools.setup(
    name="django-blip",
    version="0.0.1",
    description="Python package to intercept all external api calls during django test.",
    long_description="",
    author="Abhinav Prakash",
    author_email="abhinavsp0730@gmail.com",
    license="MIT License",
    package_dir={"": "src"},
    packages=setuptools.find_namespace_packages(where="src"),
    zip_safe=False,
    install_requires=["httpretty>=1.1.4"],
    python_requires=">=3.9",
)
